/*****************************************************************************
 *   rtc.c:  Realtime clock C file for Philips LPC214x Family Microprocessors
 *
 *   Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2005.10.01  ver 1.00    Prelimnary version, first Release
 *
*****************************************************************************/

/* modified by Martin Thomas

	- Option to use the 32kHz-XTAL as clock-source
	
	  from LPC2138 errata (hopefully taken into account correclty here)
Note.1: Increased power consumption from battery while RTC is running from the 
	main 3.3V supply
Introduction: The RTC is powered by its own power supply pin, 
	Vbat, which can be connected to a battery or to
	the same 3.3 volt supply used by the rest of the device.
Problem: If the VBAT is connected to an external battery, RTC 
	will consume more power from the battery if the core is 
	running and the selected clock sourceis the prescaler.
Work-around: Switch the clock source such that the RTC 
	takes the clock from the 32 KHz oscillator that is
	connected to the RTCX1 and RTCX2 pins. After initialization 
	of the RTC, clear the PCRTC bit in the PCONP register to 
	switch off the peripheral clock (pclk) to RTC. Any further 
	writes to the RTC would require this bit to be set. This 
	will reduce the power consumed from VBAT.
	
	- global second_tick handled as counter increment-interrupt demo
	- handle increment- and alarm-interrupt in ISR
	- reset both interrupt-flags on "start"

*/

#include "LPC214x.h"                        /* LPC21xx definitions */
#include "type.h"
#include "irq.h"
#include "timer.h"
#include "rtc.h"

volatile DWORD alarm_on = 0;
volatile DWORD second_tick;

static inline void RTC_connect(void)
{
#ifdef RTC_USE_32KHZ	
	// CCR = (CCR_CLKEN|CCR_CLKSRC);
	if ( CCR & CCR_CLKSRC) {
		PCONP |= (1UL<<9);
	}
#endif
}

static inline void RTC_disconnect(void)
{
#ifdef RTC_USE_32KHZ	
	// CCR = (CCR_CLKEN|CCR_CLKSRC);
	if ( CCR & CCR_CLKSRC) {
		PCONP &= ~(1UL<<9);
	}
#endif
}

/*****************************************************************************
** Function name:		RTCHandler
**
** Descriptions:		RTC interrupt handler, it executes based on the
**				the alarm setting
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void RTCHandler (void) __irq 
{  
	DWORD myILR = ILR;

	if ( myILR & ILR_RTCALF ) {
		/* alarm interrupt */
		ILR |= ILR_RTCALF;	/* clear interrupt flag */
		IENABLE;			/* handles nested interrupt */
		alarm_on = 1;
		IDISABLE;
    }
	
	if ( myILR & ILR_RTCCIF ) {
		/* counter increment interrupt */
		ILR |= ILR_RTCCIF;
		IENABLE;
		second_tick++;
		if ( second_tick & 1 ) {
			IOSET1 = 0x00040000;
		}
		else {
			IOCLR1 = 0x00040000;
		}
		IDISABLE;
	}

    VICVectAddr = 0;		/* Acknowledge Interrupt */
}

/*****************************************************************************
** Function name:		RTCInit
**
** Descriptions:		Initialize RTC timer
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void RTCInit( void )
{
    alarm_on = 0;

    /*--- Initialize registers ---*/
    AMR = 0;
    CIIR = 0;
    CCR = 0;

    PREINT = PREINT_RTC;
    PREFRAC = PREFRAC_RTC;
	
    return;
}

/*****************************************************************************
** Function name:		RTCStart
**
** Descriptions:		Start RTC timer
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void RTCStart( void ) 
{

	/* clear interrupt-flags alarm and increment */
	ILR |= (ILR_RTCCIF | ILR_RTCALF);

#ifdef RTC_USE_32KHZ
	CCR = (CCR_CLKEN|CCR_CLKSRC);
	RTC_disconnect();
#else
    /*--- Start RTC counters ---*/
    CCR |= CCR_CLKEN;
#endif
	
    return;
}

/*****************************************************************************
** Function name:		RTCStop
**
** Descriptions:		Stop RTC timer
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void RTCStop( void )
{   
	/*--- Stop RTC counters ---*/
    CCR &= ~CCR_CLKEN;
	
    return;
} 

/*****************************************************************************
** Function name:		RTC_CTCReset
**
** Descriptions:		Reset RTC clock tick counter
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void RTC_CTCReset( void )
{   
    /*--- Reset CTC ---*/
    CCR |= CCR_CTCRST;
    return;
}

/*****************************************************************************
** Function name:		RTCSetTime
**
** Descriptions:		Setup RTC timer value
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void RTCSetTime( RTCTime Time ) 
{
	RTC_connect();
    
	SEC = Time.RTC_Sec;
    MIN = Time.RTC_Min;
    HOUR = Time.RTC_Hour;
    DOM = Time.RTC_Mday;
    DOW = Time.RTC_Wday;
    DOY = Time.RTC_Yday;
    MONTH = Time.RTC_Mon;
    YEAR = Time.RTC_Year;    
	
	RTC_disconnect();
	
    return;
}

/*****************************************************************************
** Function name:		RTCSetAlarm
**
** Descriptions:		Initialize RTC timer
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void RTCSetAlarm( RTCTime Alarm ) 
{   
	RTC_connect();

    ALSEC = Alarm.RTC_Sec;
    ALMIN = Alarm.RTC_Min;
    ALHOUR = Alarm.RTC_Hour;
    ALDOM = Alarm.RTC_Mday;
    ALDOW = Alarm.RTC_Wday;
    ALDOY = Alarm.RTC_Yday;
    ALMON = Alarm.RTC_Mon;
    ALYEAR = Alarm.RTC_Year;

	RTC_disconnect();

    return;
}

/*****************************************************************************
** Function name:		RTCGetTime
**
** Descriptions:		Get RTC timer value
**
** parameters:			None
** Returned value:		The data structure of the RTC time table
** 
*****************************************************************************/
RTCTime RTCGetTime( void ) 
{
    RTCTime LocalTime;
    
    LocalTime.RTC_Sec = SEC;
    LocalTime.RTC_Min = MIN;
    LocalTime.RTC_Hour = HOUR;
    LocalTime.RTC_Mday = DOM;
    LocalTime.RTC_Wday = DOW;
    LocalTime.RTC_Yday = DOY;
    LocalTime.RTC_Mon = MONTH;
    LocalTime.RTC_Year = YEAR;
    return LocalTime;    
}

/*****************************************************************************
** Function name:		RTCSetAlarmMask
**
** Descriptions:		Set RTC timer alarm mask
**
** parameters:			Alarm mask setting
** Returned value:		None
** 
*****************************************************************************/
void RTCSetAlarmMask( DWORD AlarmMask ) 
{
	RTC_connect();
	
    /*--- Set alarm mask ---*/    
    AMR = AlarmMask;

	RTC_disconnect();

    return;
}

/*****************************************************************************
**                            End Of File
******************************************************************************/

