/*****************************************************************************
 *   rtctest.c:  main C entry file for Philips LPC214x Family Microprocessors
 *
 *   Original Source Copyright(C) 2006, Philips Semiconductor
 *   All rights reserved.
 *
 *   Modifications & Extensions Copyright(C) 2006, Martin Thomas 
 *   Also reserving my rights - for whatever this means for
 *   such a rather trivial example
 *
 *   History
 *   2005.10.01  ver 1.00    Prelimnary version, first Release
 *
 *   20. Sept. 2006 - "ver 2.00" - by Martin Thomas (not Philips/EnIxPi)
 *
 *****************************************************************************/

/* 
	Modified and extended by Martin Thomas
	<http://www.siwawi.arubi.uni-kl.de/avr_projects>
	
	(Not much left from the original Philips - aeh - NXP code.
	 so don't ask the Philips if it does not work for you)
	
	- RTC alarm and counter-intcrement interrupt
	- 32kHz XTAL als RTC clock-source (config-option see rtc.h)
	- power-down and idle-mode example
	- LED0 blinks @ 1Hz driven from value updated in RTC-ISR / counter
	- LED1 blinks @ 1Hz by comparting the controller's SEC-register
	- LED2 blinks @ 1Hz set inside RTC-ISR/counter
	- LED3 indicates Power-Down and Idle-Mode (on when sleeping)
	- LED4-7 are turned on when alarm occured

	(LEDs as connected on the Keil MCB2130-board, high-active,
	Schematic available at keil.com)
	
	In case your JTAG-interface can not connect to the target
	after programming this application:
	(a delay-loop has been included in main() to avoid this)
	- Press "BSL"-Button (INT1 on LPC2130) and keep pressed
	- Press Reset-Button
	- (the device is in Bootloader-Mode now and can be accessed
	   thru JTAG - as long as you did not active "Code Protection")
	- Start your JTAG-software (keep BSL pressed since there might
	  be an hardware-reset issued thru the JTAG-connector which 
	  would terminate BSL)
*/

#include "LPC214x.h"                        /* LPC21xx definitions */
#include "type.h"
#include "irq.h"
#include "timer.h"
#include "rtc.h"

// extern DWORD alarm_on; - moved to rtc.h

RTCTime local_time, alarm_time, current_time;

/*****************************************************************************
**   Main Function  main()
******************************************************************************/
int main (void)
{
	DWORD oldsec, oldsec_direct;
	volatile DWORD i;

	// delay-loop to let JTAG-hardware "catch-up"
	for (i=0; i<0x00800000; i++) {
		;
	}

	init_VIC();
	
	/* Initialize RTC module */
	RTCInit();
	
	local_time.RTC_Sec = 55;
	local_time.RTC_Min = 0;
	local_time.RTC_Hour = 0;
	local_time.RTC_Mday = 8;
	local_time.RTC_Wday = 3;
	local_time.RTC_Yday = 8;		/* "current date" 12/08/2005 00:00:55*/
	local_time.RTC_Mon = 12;
	local_time.RTC_Year = 2005;
	RTCSetTime( local_time );		/* Set local time */
	
	alarm_time.RTC_Sec = 0;
	alarm_time.RTC_Min = 1;
	alarm_time.RTC_Hour = 0;
	alarm_time.RTC_Mday = 8;
	alarm_time.RTC_Wday = 3;
	alarm_time.RTC_Yday = 8;		/* alarm date 12/08/2005 00:01:00 */
	alarm_time.RTC_Mon = 12;
	alarm_time.RTC_Year = 2005;
	RTCSetAlarm( alarm_time );		/* set alarm time */
	
	/* install RTC timer handler mainly for alarm control */
	if ( install_irq( RTC_INT, (void *)RTCHandler ) == FALSE )
	{
		while ( 1 );		/* very bad happened */
	}
	
	/* only "minute match" will generate in alarm interrupt - all others masked */
	RTCSetAlarmMask(AMRSEC/*|AMRMIN*/|AMRHOUR|AMRDOM|AMRDOW|AMRDOY|AMRMON|AMRYEAR);
	/* Change in Second will generate an increment interrupt */
	CIIR = IMSEC;
	
	IOCLR1 = 0x00FF0000;
	IOSET1 = 0x00020000;
	IODIR1 = 0x00FF0000;		/* P1.16..23 defined as Outputs */

	RTCStart();
	
	oldsec_direct = SEC;
	oldsec = second_tick;

	while (1) 
	{					/* Loop forever */
		if ( second_tick != oldsec ) {
			oldsec = second_tick;
			if ( oldsec & 1 ) {
				IOSET1 = 0x00010000;
			}
			else {
				IOCLR1 = 0x00010000;
			}
		}
		if ( SEC != oldsec_direct ) {
			oldsec_direct = SEC;
			if ( oldsec_direct & 1 ) {
				IOCLR1 = 0x00020000;
			}
			else {
				IOSET1 = 0x00020000;
			}
		}
		if ( alarm_on != 0 ) {
			alarm_on = 0;
			/* Get current time when alarm is on */
			current_time = RTCGetTime();
			/* light some leds on alarm */
			IOSET1 = 0x00F00000;
		}
		
		IOSET1 = 0x00080000;     // indicate power-save

		i=1; // dummy-writes (i is a "volatile")
		i=2;

#ifdef RTC_USE_32KHZ
#warning "Using Power-Down"
		INTWAKE |= (1<<15);     // RTCWAKE(15) - enable RTC wakeup from Power-Down
		PCON |= (1<<1);         // PD(1) - Power Down
		/* snooze ... */
		PCON &= ~(1<<15);       // clear PD
		INTWAKE &= ~(1<<15);    // clear RTCWAKE
		// "Power-down mode automatically turns off and disconnects the PLL."
		// -> setup PLL and re-connect
		PLLCFG  = 0x00000024;   // sorry, hardcoded "magic" for now see Startup.S 
		PLLCON  = 1;            // PLLE(0) - enable PLL
		PLLFEED = 0xAA;         // make e happen 1
		PLLFEED = 0x55;         // make e happen 2
		while((PLLSTAT & (1UL<<10)) == 0) {
			;                   // wait for PLL locked
		}
		PLLCON  = 3;            // PLLE(0) & PLLC(1) - enable & connect
		PLLFEED = 0xAA;         // make e&c happen 1
		PLLFEED = 0x55;         // make e&c happen 2
#else
#warning "Using Idle"
		PCON |= (1<<0);   // Idle(0)
		// snooze ...
		PCON &= ~(1<<0);  // clear idle bit
#endif	
		IOCLR1 = 0x00080000; // indicate active
	}
	
	return 0; /* never reached */
}

/******************************************************************************
**                            End Of File
******************************************************************************/
